#include <flext.h>
#include "Dfm1aProcessing.h"

#if !defined(FLEXT_VERSION) || (FLEXT_VERSION < 400) 
#error You need at least flext version 0.4.0
#endif

class dfm1a:public flext_dsp {
  FLEXT_HEADER(dfm1a, flext_dsp)
  public:
    dfm1a(int argc, t_atom *argv);

  protected:
    virtual void m_signal(int n, float *const *in, float *const *out);

  private:
    Dfm1aState df;

    float m_frequency;
    float m_resonance;
    float m_inputGain;
    float m_type;
    float m_noiseLevel;

    void setFrequency(float value);
    void setResonance(float value);
    void setInputGain(float value);
    void setType(float value);
    void setNoiseLevel(float value);

    FLEXT_CALLBACK_F(setFrequency)
    FLEXT_CALLBACK_F(setResonance)
    FLEXT_CALLBACK_F(setInputGain)
    FLEXT_CALLBACK_F(setType)
    FLEXT_CALLBACK_F(setNoiseLevel)
};

FLEXT_NEW_DSP_V("dfm1a~", dfm1a)

dfm1a::dfm1a(int argc, t_atom* argv) {
  m_frequency = 440.0f;
  m_resonance = 0.7f;
  m_inputGain = 1.0f;
  m_type = 0.0f;
  m_noiseLevel = 0.001f;
 
  switch(argc) {
    default:
    case 5:
      m_noiseLevel = GetFloat(argv[4]);
    case 4:
      m_type = GetFloat(argv[3]);
    case 3:
      m_inputGain = GetFloat(argv[2]);
    case 2:
      m_resonance = GetFloat(argv[1]);
    case 1:
      m_frequency = GetFloat(argv[0]);
    case 0:
      ;
  }

  Dfm1aProcessing::initialize(&df);

  AddInSignal("audio in");
  AddInSignal("frequency mod");
  AddInFloat("frequency");
  AddInFloat("resonance");
  AddInFloat("input gain");
  AddInFloat("type");
  AddInFloat("noise level");

  AddOutSignal();

  FLEXT_ADDMETHOD(2, setFrequency);
  FLEXT_ADDMETHOD(3, setResonance);
  FLEXT_ADDMETHOD(4, setInputGain);
  FLEXT_ADDMETHOD(5, setType);
  FLEXT_ADDMETHOD(6, setNoiseLevel);
}

void dfm1a::m_signal(int n, float *const *in, float *const *out) {
  float *in1 = in[0];
  float *mod1 = in[1];
  float* out1 = out[0];
  Dfm1aProcessing::filter(m_inputGain, m_frequency, m_resonance, m_type, m_noiseLevel, in1, out1, mod1, n, Samplerate(), &df);
}

void dfm1a::setFrequency(float value) {
  m_frequency = value;
}

void dfm1a::setResonance(float value) {
  m_resonance = value;
}

void dfm1a::setInputGain(float value) {
  m_inputGain = value;
}

void dfm1a::setType(float value) {
  m_type = value;
}

void dfm1a::setNoiseLevel(float value) {
  m_noiseLevel = value;
}
